from GameVisionTargetingModel.models.classification.map.v1.setup.middle_layer\
    import                          \
    generate_first_middle_layer,    \
    generate_second_middle_layer,   \
    generate_third_middle_layer,    \
    generate_decision_middle_layer


def generate_middle_layer(
        layers: list
):
    generate_first_middle_layer(
        layers
    )

    generate_second_middle_layer(
        layers
    )

    generate_third_middle_layer(
        layers
    )

    generate_decision_middle_layer(
        layers
    )



from keras.models \
    import Model

from GameVisionTargetingModel.models.classification \
    import MapClassifierVersion1

map_classifier_model = None


def get_map_classifier_model():
    global map_classifier_model

    if map_classifier_model is None:
        set_map_classifier_model(
            MapClassifierVersion1()
        )

    return map_classifier_model


def set_map_classifier_model(
        value: Model
):
    global map_classifier_model
    map_classifier_model = value


def delete_of_map_classifier() -> None:
    global map_classifier_model
    del map_classifier_model




from keras.models \
    import Sequential

from tensorflow \
    import get_logger

from GameVisionTargetingModel.models.classification.map.v1.layers \
    import generate_layers_for_map_classifier_v1

from keras.losses \
    import SparseCategoricalCrossentropy

from keras.metrics              \
    import                      \
    Accuracy,                   \
    SparseCategoricalAccuracy

from logging \
    import ERROR


def tensorflow_settings():
    get_logger().setLevel(
        ERROR
    )


class MapClassifier(
    Sequential
):
    def __init__(
            self,
            optional_preprocessing: bool = False
    ):
        super().__init__(
            generate_layers_for_map_classifier_v1(
                optional_preprocessing
            )
        )

        tensorflow_settings()

        self.setup()

    def setup(self):
        self.compile(
            loss=SparseCategoricalCrossentropy(
                from_logits=True,
                ignore_class=None
            ),
            metrics=[
                Accuracy('accuracy'),
            ],
            optimizer='adam',
            jit_compile=True
        )




from GameVisionTargetingModel.models.classification.map.v1.setup.setup_of_input_layer \
    import generate_input_layer

from GameVisionTargetingModel.models.classification.map.v1.setup.setup_of_middle_layer \
    import generate_middle_layer

from GameVisionTargetingModel.models.classification.map.v1.setup.setup_of_output_layer \
    import generate_output_layer

from GameVisionTargetingModel.models.classification.map.v1.setup.setup_of_preprocessing \
    import generate_preprocessing_layers



from GameVisionTargetingModel.models.classification.map.v1.setup \
    import                  \
    generate_input_layer,   \
    generate_output_layer,  \
    generate_middle_layer,  \
    generate_preprocessing_layers


def generate_layers_for_map_classifier_v1(
        addition_of_preprocessing: bool = False
) -> list:
    return_layers: list = []

    generate_input_layer(
        return_layers
    )

    if addition_of_preprocessing:
        generate_preprocessing_layers(
            return_layers
        )

    generate_middle_layer(
        return_layers
    )

    generate_output_layer(
        return_layers
    )

    return return_layers



from GameVisionTargetingModel.setup_package \
    import default_settings

from GameVisionTargetingModel.variables \
    import                      \
    get_width,                  \
    get_height,                 \
    get_channels,               \
    set_width,                  \
    set_height,                 \
    set_channels,               \
    set_number_of_labels,       \
    set_last_layer_multiplier,  \
    get_last_layer_multiplier,  \
    get_number_of_labels


def test_defaults() -> None:
    default_settings()


def test_height() -> None:
    height = 256

    set_height(height)

    assert get_height() == height


def test_width() -> None:
    width = 128

    set_width(width)

    assert get_width() == width


def test_channels() -> None:
    channels = 9

    set_channels(channels)

    assert get_channels() == channels


def test_number_labels() -> None:
    n_labels = 20

    set_number_of_labels(n_labels)

    assert get_number_of_labels() == n_labels


def test_last_layers() -> None:
    multiplier = 2

    set_last_layer_multiplier(multiplier)

    assert get_last_layer_multiplier() == multiplier



from GameVisionTargetingModel.variables \
    import \
    set_channels, \
    set_width, \
    set_height, \
    set_last_layer_multiplier, \
    set_number_of_labels


def default_settings() -> None:
    setup_input_variables()
    setup_output_variables()


def setup_input_variables(
        width: int = 512,
        height: int = 512,
        channels: int = 3
):
    set_width(
        width
    )

    set_height(
        height
    )

    set_channels(
        channels
    )


def setup_output_variables(
        last_layer_multiplier: int = 8,
        number_of_labels: int = 1000,
):
    set_last_layer_multiplier(
        last_layer_multiplier
    )

    set_number_of_labels(
        number_of_labels
    )



from keras.layers \
    import \
    Flatten, \
    Dense

from GameVisionTargetingModel.variables\
    import \
    get_number_of_labels, \
    get_last_layer_multiplier


def generate_output_layer(
    layers: list
):
    size_of_decision_layer: int = get_number_of_labels() * get_last_layer_multiplier()

    layers.append(
        Flatten()
    )

    layers.append(
        Dense(
            size_of_decision_layer
        )
    )

    layers.append(
        Dense(
            get_number_of_labels()
        )
    )
